"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const errors_1 = require("./errors");
const execute = (rules, scripts, warning, configFix = false) => {
    /**
     * keep track of everything
     */
    const issues = [];
    let issuesFixed = 0;
    const patchScripts = (newScripts) => {
        issuesFixed++;
        scripts = newScripts;
    };
    /**
     * execution functions
     */
    const executeObjectRule = ({ validate, message, name, fix }) => {
        if (typeof validate !== "function") {
            throw new errors_1.ValidationFunctionInvalidError(`Rule validation function is not a function (${name})`);
        }
        const validationResult = validate(scripts);
        const fixable = typeof fix === "function";
        /**
         * validate
         */
        const valid = typeof validationResult === "boolean"
            ? validationResult
            : validationResult.length < 1;
        if (valid) {
            return;
        }
        /**
         * warn of invalidity
         */
        const warningMessage = typeof validationResult === "boolean"
            ? `${message} (${name})`
            : (0, utils_1.makeMessage)(`${message} (${name})`, {
                names: validationResult.join(", "),
            });
        /**
         * potentially fix
         */
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (configFix && fixable && fix) {
            patchScripts(fix(scripts));
            return;
        }
        /**
         * keep track of everything
         */
        issues.push({
            name,
            affected: undefined,
            type: "warning",
            message: warningMessage,
        });
        if (typeof warning === "function") {
            warning(warningMessage, validationResult);
        }
    };
    const executeEntryRule = ({ validate, message, name, fix }) => {
        if (typeof validate !== "function") {
            throw new errors_1.ValidationFunctionInvalidError(name);
        }
        const fixable = typeof fix === "function";
        const pairs = Object.entries(scripts);
        /**
         * iterate all the scripts
         */
        pairs.forEach(([key, value]) => {
            const valid = validate(key, value, scripts);
            if (valid) {
                return;
            }
            const warningMessage = (0, utils_1.makeMessage)(`${message} (${name})`, {
                name: key,
            });
            /**
             * potentially fix
             */
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            if (configFix && fixable && fix) {
                const [toKey, fixedValue] = fix(key, value);
                const fixedScripts = (0, utils_1.patchScriptObjectEntry)(scripts, key, toKey, fixedValue);
                patchScripts(fixedScripts);
                return;
            }
            /**
             * keep track of everything
             */
            issues.push({
                name,
                affected: key,
                type: "warning",
                message: warningMessage,
            });
            if (typeof warning === "function") {
                warning(warningMessage, key);
            }
        });
    };
    /**
     * and go!
     */
    rules.forEach((rule) => {
        if (rule.isObjectRule) {
            executeObjectRule(rule);
        }
        else {
            executeEntryRule(rule);
        }
    });
    return [issues, scripts, issuesFixed];
};
exports.default = execute;
